@@ -24,9 +24,12 @@
 #include "../objects/normalpoint.h"
 #include "../misc/i18n.h"
 #include "../misc/coordinate_system.h"
+#include "../misc/argsparser.h"
+#include "../modes/selectionmode.h"
 
 #include <klineeditdlg.h>
 #include <kmessagebox.h>
+#include <kdebug.h>
 
 void AddFixedPointAction::slotActivated()
 {
@@ -36,7 +39,7 @@ void AddFixedPointAction::slotActivated()
   while ( ! done )
   {
     QString s = KLineEditDlg::getText(
-      i18n( "Fixed point" ), i18n( "Enter the coordinates for the new point.." ) +
+      i18n( "Fixed Point" ), i18n( "Enter the coordinates for the new point." ) +
       QString::fromUtf8("\n") + mdoc->coordinateSystem().coordinateFormatNotice(),
       QString::null, &ok, mdoc->widget() );
     if ( ! ok ) return;
@@ -56,9 +59,39 @@ void AddFixedPointAction::slotActivated()
 AddFixedPointAction::AddFixedPointAction( KigDocument* doc,
                                           const QIconSet& icon,
                                           KActionCollection* coll )
-  : KAction( i18n( "Point by coordinates" ), icon, 0, 0, 0, coll,
+  : KAction( i18n( "Point by Coordinates" ), icon, 0, 0, 0, coll,
              "objects_new_point_xy" ),
     mdoc( doc )
 {
-    setToolTip( i18n( "Construct a point by entering its coordinates..." ) );
+    setToolTip( i18n( "Construct a point by entering its coordinates." ) );
 }
+
+TestAction::TestAction( KigDocument* doc, const QIconSet& icon,
+                        KActionCollection* parent )
+  : KAction( i18n( "Test Stuff!" ), icon, 0, 0, 0, parent,
+             "test_stuff" ),
+    mdoc( doc )
+{
+}
+
+const struct ArgParser::spec testspec[] =
+{
+  { Object::AbstractLineT, 2 },
+  { Object::PointT, 1 }
+};
+
+void TestAction::slotActivated()
+{
+  ArgParser checker( testspec, 2 );
+  StandAloneSelectionMode mode( checker, *mdoc );
+  mode.run( mdoc->mode() );
+  Objects sel = mode.selection();
+  sel = checker.parse( sel );
+  kdDebug() << k_funcinfo << endl
+            << sel.size() << endl;
+  for ( uint i = 0; i < sel.size(); ++i )
+  {
+    kdDebug() << sel[i]->vBaseTypeName() << endl;
+  }
+}
+
