@@ -32,6 +32,8 @@
 #include "../objects/conic.h"
 #include "../objects/coniclines.h"
 #include "../objects/conicsextra.h"
+#include "../objects/cubic.h"
+#include "../objects/transform.h"
 #include "../objects/segment.h"
 #include "../objects/normalpoint.h"
 #include "../objects/midpoint.h"
@@ -47,7 +49,7 @@
 #include "../objects/ray.h"
 #include "../objects/coordproppoint.h"
 #include "../objects/rotatedpoint.h"
-#include "../objects/arc.h"
+#include "../objects/angle.h"
 #include "../misc/type.h"
 #include "../misc/coordinate_system.h"
 #include "../misc/calcpaths.h"
@@ -58,6 +60,7 @@
 #include <kinstance.h>
 #include <kfiledialog.h>
 #include <kaction.h>
+#include <kapplication.h>
 #include <ktoolbar.h>
 #include <kmainwindow.h>
 #include <kstdaction.h>
@@ -71,6 +74,9 @@
 
 #include <qfile.h>
 #include <qtimer.h>
+#if QT_VERSION >= 0x030100
+#include <qeventloop.h>
+#endif
 
 #include <algorithm>
 
@@ -120,7 +126,7 @@ KigDocument::KigDocument( QWidget *parentWidget, const char *,
 
   setModified (false);
 
-  mMode = new NormalMode( this );
+  mMode = new NormalMode( *this );
 }
 
 void KigDocument::setupActions()
@@ -135,50 +141,53 @@ void KigDocument::setupActions()
 
   tmp = l->loadIcon( "delete", KIcon::User);
   aDeleteObjects = new KAction(
-      i18n("Delete objects"), "editdelete", Key_Delete, this,
+      i18n("Delete Objects"), "editdelete", Key_Delete, this,
       SLOT(deleteObjects()), actionCollection(), "delete_objects");
   aDeleteObjects->setToolTip(i18n("Delete the selected objects"));
 
   tmp = l->loadIcon( "stop", KIcon::Toolbar);
   aCancelConstruction = new KAction(
-      i18n("Cancel construction"), tmp, Key_Escape, this,
+      i18n("Cancel Construction"), tmp, Key_Escape, this,
       SLOT(cancelConstruction()), actionCollection(), "cancel_construction");
   aCancelConstruction->setToolTip(
       i18n("Cancel the construction of the object being constructed"));
   aCancelConstruction->setEnabled(false);
 
   aShowHidden = new KAction(
-    i18n("Unhide all"), 0, this, SLOT( showHidden() ),
+    i18n("Unhide All"), 0, this, SLOT( showHidden() ),
     actionCollection(), "edit_unhide_all");
   aShowHidden->setToolTip(i18n("Show all hidden objects"));
   aShowHidden->setEnabled( true );
 
   tmp = l->loadIcon("gear", KIcon::Toolbar);
   aNewMacro = new KAction(
-    i18n("New macro"), tmp, 0, this, SLOT(newMacro()),
+    i18n("New Macro..."), tmp, 0, this, SLOT(newMacro()),
     actionCollection(), "macro_action");
   aNewMacro->setToolTip(i18n("Define a new macro"));
 
   aConfigureTypes = new KAction(
     i18n("Types Manager"), 0, this, SLOT(editTypes()),
     actionCollection(), "types_edit");
-  aConfigureTypes->setToolTip(i18n("Manage macro types.."));
+  aConfigureTypes->setToolTip(i18n("Manage macro types."));
 
 //   tmp = l->loadIcon( "window_fullscreen", KIcon::User );
 //   aFullScreen = new KAction(
-//     i18n( "Full screen" ), tmp, 0, this, SLOT( startKiosk() ),
+//     i18n( "Full Screen" ), tmp, 0, this, SLOT( startKiosk() ),
 //     actionCollection(), "view_fullscreen" );
 //   aFullScreen->setToolTip( i18n( "View this document full-screen." ) );
 
   tmp = l->loadIcon( "pointxy", KIcon::User );
   aFixedPoint = new AddFixedPointAction( this, tmp, actionCollection() );
+
+  tmp = l->loadIcon( "new", KIcon::User );
+  (void) new TestAction( this, tmp, actionCollection() );
 };
 
 void KigDocument::setupTypes()
 {
   if ( Object::types().empty() )
   {
-    Object::addBuiltinType( new TStdType<Segment> );
+    Object::addBuiltinType( new TStdType<SegmentAB> );
     Object::addBuiltinType( new TMultiType<LineConicAsymptotes> );
     Object::addBuiltinType( new TStdType<ConicBAAP> );
     Object::addBuiltinType( new TStdType<ConicBDFP> );
@@ -188,24 +197,34 @@ void KigDocument::setupTypes()
     Object::addBuiltinType( new TStdType<LineParallel> );
     Object::addBuiltinType( new TStdType<LineRadical> );
     Object::addBuiltinType( new TMultiType<LineConicRadical> );
+    Object::addBuiltinType( new TStdType<PointTransform> );
+    Object::addBuiltinType( new TStdType<LineTransform> );
+    Object::addBuiltinType( new TStdType<SegmentTransform> );
+    Object::addBuiltinType( new TStdType<RayTransform> );
+    Object::addBuiltinType( new TStdType<CircleTransform> );
+    Object::addBuiltinType( new TStdType<ConicTransform> );
     Object::addBuiltinType( new TStdType<CircleBCP> );
     Object::addBuiltinType( new TStdType<CircleBTP> );
     Object::addBuiltinType( new TStdType<EllipseBFFP> );
     Object::addBuiltinType( new TStdType<HyperbolaBFFP> );
-//     Object::addBuiltinType( new TStdType<ConicBDF> );
     Object::addBuiltinType( new TStdType<ConicB5P> );
+    Object::addBuiltinType( new TStdType<CubicB9P> );
+    Object::addBuiltinType( new TStdType<CubicNodeB6P> );
+    Object::addBuiltinType( new TStdType<CubicCuspB4P> );
+    Object::addBuiltinType( new TStdType<CubicTransform> );
     Object::addBuiltinType( new TStdType<ParabolaBTP> );
     Object::addBuiltinType( new TStdType<EquilateralHyperbolaB4P> );
     Object::addBuiltinType( new TStdType<MidPoint> );
     Object::addBuiltinType( new TStdType<IntersectionPoint> );
     Object::addBuiltinType( new TMultiType<CircleLineIntersectionPoint> );
     Object::addBuiltinType( new TMultiType<ConicLineIntersectionPoint> );
+    Object::addBuiltinType( new TMultiType<CubicLineIntersectionPoint> );
     Object::addBuiltinType( new TStdType<TranslatedPoint> );
     Object::addBuiltinType( new TStdType<MirrorPoint> );
     Object::addBuiltinType( new TStdType<Locus> );
     Object::addBuiltinType( new TStdType<Vector> );
-    Object::addBuiltinType( new TStdType<Ray> );
-    Object::addBuiltinType( new TStdType<Arc> );
+    Object::addBuiltinType( new TStdType<RayAB> );
+    Object::addBuiltinType( new TStdType<Angle> );
     Object::addBuiltinType( new TStdType<LineDirectrix> );
     Object::addBuiltinType( new TStdType<LinePolar> );
     Object::addBuiltinType( new TStdType<PointPolar> );
@@ -293,7 +312,7 @@ bool KigDocument::openFile()
               "always ask me nicely on mailto:devriese@kde.org "
               "or do the work yourself and send me a patch."
           ).arg(mimeType->name()),
-        i18n( "Format not supported" )
+        i18n( "Format not Supported" )
         );
     return false;
   };
@@ -309,7 +328,7 @@ bool KigDocument::openFile()
         "valid, and you think Kig should be able to open it, you can try to "
         "send me a copy of the file and ask me nicely to check it out.  If "
         "you want more certain results, you can always do the work yourself "
-        "( since Kig is Free Software ), and send me a patch..."
+        "( since Kig is Free Software ), and send me a patch."
                             ) );
     return false;
   };
@@ -352,7 +371,7 @@ bool KigDocument::saveFile()
               "always ask me nicely on mailto:devriese@kde.org "
               "or do the work yourself and send me a patch."
           ).arg(mimeType->name()),
-        i18n( "Format not supported" )
+        i18n( "Format not Supported" )
         );
     return false;
   };
@@ -369,7 +388,7 @@ bool KigDocument::saveFile()
     // i know i need to change the enum value name, but this means
     // that the file could not be opened...
     KMessageBox::sorry( m_widget,
-                        i18n( "The file \"%1\" could not be opened.  Please check if the file permissions are set correctly..." )
+                        i18n( "The file \"%1\" could not be opened.  Please check if the file permissions are set correctly." )
                         .arg( m_file ) );
     return false;
 
@@ -386,7 +405,7 @@ bool KigDocument::saveFile()
               "always ask me nicely on mailto:devriese@kde.org "
               "or do the work yourself and send me a patch."
           ).arg(mimeType->name()),
-        i18n( "Format not supported" )
+        i18n( "Format not Supported" )
         );
   };
   return false;
@@ -666,7 +685,7 @@ bool KigDocument::internalSaveAs()
   else if ( QFileInfo( file_name ).exists() )
   {
     int ret = KMessageBox::warningYesNo( m_widget,
-                                         i18n( "The file \"%1\" already exists.  Do you wish to overwrite it ?" )
+                                         i18n( "The file \"%1\" already exists.  Do you wish to overwrite it?" )
                                          .arg( file_name ) );
     if ( ret != KMessageBox::Yes )
     {
@@ -681,3 +700,28 @@ KigView* KigDocument::mainWidget()
 {
   return m_widget;
 }
+
+void KigDocument::runMode( KigMode* m )
+{
+  KigMode* prev = mMode;
+
+  setMode( m );
+
+#if QT_VERSION >= 0x030100
+  (void) kapp->eventLoop()->enterLoop();
+#else
+  (void) kapp->enter_loop();
+#endif
+
+  setMode( prev );
+}
+
+void KigDocument::doneMode( KigMode* d )
+{
+  assert( d == mMode );
+#if QT_VERSION >= 0x030100
+  kapp->eventLoop()->exitLoop();
+#else
+  kapp->exit_loop();
+#endif
+}
