@@ -1,21 +1,21 @@
 /**
- This file is part of Kig, a KDE program for Interactive Geometry...
- Copyright (C) 2002  Dominique Devriese <devriese@kde.org>
+   This file is part of Kig, a KDE program for Interactive Geometry...
+   Copyright (C) 2002  Dominique Devriese <devriese@kde.org>
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
- (at your option) any later version.
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
- USA
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+   USA
 **/
 
 #include "kigpainter.h"
@@ -23,6 +23,7 @@
 #include "../kig/kig_view.h"
 #include "../objects/object.h"
 #include "../objects/conic.h"
+#include "../objects/cubic.h"
 #include "common.h"
 #include "coordinate_system.h"
 
@@ -34,7 +35,8 @@
 KigPainter::KigPainter( const ScreenInfo& si, QPaintDevice* device, bool no )
   : mP ( device ),
     msi( si ),
-    mNeedOverlay( no )
+    mNeedOverlay( no ),
+    overlayenlarge( 0 )
 {
 }
 
@@ -113,7 +115,7 @@ void KigPainter::textOverlay( const QRect& r, const QString s, int textFlags, in
 }
 
 const Rect KigPainter::boundingRect( const Rect& r, const QString s,
-                                      int f, int l ) const
+                                     int f, int l ) const
 {
   return fromScreen( mP.boundingRect( toScreen( r ), f, s, l ) );
 };
@@ -133,6 +135,7 @@ void KigPainter::setStyle( const PenStyle c )
 void KigPainter::setWidth( const uint c )
 {
   width = c;
+  if (c > 0) overlayenlarge = c - 1;
   mP.setPen( QPen( color, width, style ) );
 }
 
@@ -171,11 +174,11 @@ void KigPainter::drawPolygon( const std::vector<Coordinate>& pts, bool winding,
   QPointArray t( pts.size() );
   int c = 0;
   for( std::vector<Coordinate>::const_iterator i = pts.begin(); i != pts.end(); ++i )
-    {
-      sr.setContains( *i );
-      QPoint tt = toScreen (*i);
-      t.putPoints( c++, 1, tt.x(), tt.y() );
-    };
+  {
+    sr.setContains( *i );
+    QPoint tt = toScreen (*i);
+    t.putPoints( c++, 1, tt.x(), tt.y() );
+  };
   mP.drawPolygon( t, winding, index, npoints );
   mOverlay.push_back( toScreen( sr ) );
 }
@@ -185,7 +188,8 @@ Rect KigPainter::window()
   return msi.shownRect();
 }
 
-void KigPainter::circleOverlayRecurse( const Coordinate& centre, double radius,
+void KigPainter::circleOverlayRecurse( const Coordinate& centre,
+				       double radiussq,
 				       const Rect& cr )
 {
   Rect currentRect = cr.normalized();
@@ -199,30 +203,36 @@ void KigPainter::circleOverlayRecurse( const Coordinate& centre, double radius,
   Coordinate bl = currentRect.bottomLeft();
   Coordinate c = currentRect.center();
 
-  // 1.415 should actually be 1.414...
-  double fault = currentRect.width()*1.415 + pixelWidth();
-  double radiusBig = radius + fault ;
-  double radiusSmall = radius - fault ;
+  // mp: we compute the minimum and maximum distance from the center
+  // of the circle and this rect
+  double distxmin = 0, distxmax = 0, distymin = 0, distymax = 0;
+  if ( centre.x >= tr.x ) distxmin = centre.x - tr.x;
+  if ( centre.x <= bl.x ) distxmin = bl.x - centre.x;
+  if ( centre.y >= tr.y ) distymin = centre.y - tr.y;
+  if ( centre.y <= bl.y ) distymin = bl.y - centre.y;
+  distxmax = fabs(centre.x - c.x) + currentRect.width()/2;
+  distymax = fabs(centre.y - c.y) + currentRect.height()/2;
+  // this should take into account the thickness of the line...
+  distxmin -= pixelWidth();
+  if (distxmin < 0) distxmin = 0;
+  distxmax += pixelWidth();
+  distymin -= pixelWidth();
+  if (distymin < 0) distymin = 0;
+  distymax += pixelWidth();
+  double distminsq = distxmin*distxmin + distymin*distymin;
+  double distmaxsq = distxmax*distxmax + distymax*distymax;
 
   // if the circle doesn't touch this rect, we return
   // too far from the centre
-  if (((tl - centre).length() > radiusBig) &&
-      ((tr - centre).length() > radiusBig) &&
-      ((bl - centre).length() > radiusBig) &&
-      ((br - centre).length() > radiusBig))
-    return;
+  if (distminsq > radiussq) return;
 
   // too near to the centre
-  if (((tl - centre).length() < radiusSmall) &&
-      ((tr - centre).length() < radiusSmall) &&
-      ((bl - centre).length() < radiusSmall) &&
-      ((br - centre).length() < radiusSmall))
-    return;
+  if (distmaxsq < radiussq) return;
 
   // the rect contains some of the circle
   // -> if it's small enough, we keep it
   if( currentRect.width() < overlayRectSize() ) {
-    mOverlay.push_back( toScreen( currentRect) );
+    mOverlay.push_back( toScreenEnlarge( currentRect) );
   } else {
     // this func works recursive: we subdivide the current rect, and if
     // it is of a good size, we keep it, otherwise we handle it again
@@ -230,16 +240,16 @@ void KigPainter::circleOverlayRecurse( const Coordinate& centre, double radius,
     double height = currentRect.height() / 2;
     Rect r1 ( c, -width, -height);
     r1.normalize();
-    circleOverlayRecurse(centre, radius, r1);
+    circleOverlayRecurse(centre, radiussq, r1);
     Rect r2 ( c, width, -height);
     r2.normalize();
-    circleOverlayRecurse(centre, radius, r2);
+    circleOverlayRecurse(centre, radiussq, r2);
     Rect r3 ( c, -width, height);
     r3.normalize();
-    circleOverlayRecurse(centre, radius, r3);
+    circleOverlayRecurse(centre, radiussq, r3);
     Rect r4 ( c, width, height);
     r4.normalize();
-    circleOverlayRecurse(centre, radius, r4);
+    circleOverlayRecurse(centre, radiussq, r4);
   };
 };
 
@@ -250,7 +260,7 @@ void KigPainter::circleOverlay( const Coordinate& centre, double radius )
   Coordinate bottomLeft = centre - r;
   Coordinate topRight = centre + r;
   Rect rect( bottomLeft, topRight );
-  circleOverlayRecurse ( centre , radius, rect );
+  circleOverlayRecurse ( centre , radius*radius, rect );
 }
 
 void KigPainter::segmentOverlay( const Coordinate& p1, const Coordinate& p2 )
@@ -260,8 +270,11 @@ void KigPainter::segmentOverlay( const Coordinate& p1, const Coordinate& p2 )
   // some stuff we may need:
   Coordinate p3 = p2 - p1;
   Rect border = window();
-  double length = p3.length();
-  if ( length < 10e-20 )
+//  double length = p3.length();
+  // mp: using the l-infinity distance is more natural here
+  double length = fabs(p3.x);
+  if ( fabs( p3.y ) > length ) length = fabs( p3.y );
+  if ( length < pixelWidth() )
   {
     // hopefully prevent SIGZERO's
     mOverlay.push_back( toScreen( Rect( p1, p2 ) ) );
@@ -280,17 +293,17 @@ void KigPainter::segmentOverlay( const Coordinate& p1, const Coordinate& p2 )
     Coordinate tP = p1+p3*counter;
     tR.setCenter(tP);
     if (!tR.intersects(r))
-      {
-	//kdDebug()<< "stopped after "<< counter << " passes." << endl;
-	break;
-      }
-    if (tR.intersects(border)) mOverlay.push_back( toScreen( tR ) );
-    if (++counter > 100)
-      {
-	kdError()<< k_funcinfo << "counter got too big :( " << endl;
-	break;
-      }
+    {
+      //kdDebug()<< "stopped after "<< counter << " passes." << endl;
+      break;
     }
+    if (tR.intersects(border)) mOverlay.push_back( toScreenEnlarge( tR ) );
+    if (++counter > 100)
+    {
+      kdDebug()<< k_funcinfo << "counter got too big :( " << endl;
+      break;
+    }
+  }
 }
 
 double KigPainter::overlayRectSize()
@@ -369,6 +382,19 @@ QRect KigPainter::toScreen( const Rect r ) const
   return msi.toScreen( r );
 }
 
+QRect KigPainter::toScreenEnlarge( const Rect r ) const
+{
+  if ( overlayenlarge == 0 ) return msi.toScreen( r );
+
+  QRect qr = msi.toScreen( r );
+  qr.moveBy ( -overlayenlarge, -overlayenlarge );
+  int w = qr.width();
+  int h = qr.height();
+  qr.setWidth (w + 2*overlayenlarge);
+  qr.setHeight (h + 2*overlayenlarge);
+  return qr;
+}
+
 void KigPainter::drawSimpleText( const Coordinate& c, const QString s )
 {
   int tf = AlignLeft | AlignTop | DontClip | WordBreak;
@@ -388,7 +414,7 @@ const Rect KigPainter::simpleBoundingRect( const Coordinate& c, const QString s
 }
 
 const Rect KigPainter::boundingRect( const Coordinate& c, const QString s,
-                                      int f, int l ) const
+                                     int f, int l ) const
 {
   return boundingRect( Rect( c, mP.window().right(), mP.window().top() ),
                        s, f, l );
@@ -411,7 +437,7 @@ void KigPainter::drawRay( const Coordinate& a, const Coordinate& b )
   drawSegment( a, tb );
 }
 
-void KigPainter::drawArc( const Rect& surroundingRect, int startAngle, int angle )
+void KigPainter::drawAngle( const Rect& surroundingRect, int startAngle, int angle )
 {
   mP.drawArc( toScreen( surroundingRect ), startAngle, angle );
   Rect r = surroundingRect;
@@ -423,7 +449,7 @@ void KigPainter::drawArc( const Rect& surroundingRect, int startAngle, int angle
 
 void KigPainter::drawFatPoint( const Coordinate& p )
 {
-    drawFatPoint( p, 5 * pixelWidth() );
+  drawFatPoint( p, 5 * pixelWidth() );
 }
 
 inline Coordinate conicGetCoord( double theta, double ecostheta0,
@@ -438,6 +464,18 @@ inline Coordinate conicGetCoord( double theta, double ecostheta0,
   return focus1 + rho * Coordinate( costheta, sintheta );
 };
 
+typedef std::pair<double,Coordinate> coordparampair;
+
+struct workitem
+{
+  workitem( coordparampair f, coordparampair s, Rect *o) :
+    first(f), second(s), overlay(o) {};
+  coordparampair first;
+  coordparampair second;
+  Rect		 *overlay;
+};
+//typedef std::pair<coordparampair, coordparampair> workitem;
+
 void KigPainter::drawConic( const ConicPolarEquationData& data )
 {
   // @author Maurizio Paolini wrote an original version of this, ( the
@@ -460,27 +498,43 @@ void KigPainter::drawConic( const ConicPolarEquationData& data )
   // we manage our own overlay
   bool tNeedOverlay = mNeedOverlay;
   mNeedOverlay = false;
-  Rect overlay;
+  QPen pen = mP.pen();
+  pen.setCapStyle( Qt::RoundCap );
+  mP.setPen( pen );
 
   // this stack contains pairs of Coordinates that we still need to
   // process:
-  typedef std::pair<double,Coordinate> coordparampair;
-  typedef std::pair<coordparampair, coordparampair> workitem;
   std::stack<workitem> workstack;
+  // mp: this stack contains all the generated overlays:
+  // the strategy for generating the overlay structure is the same
+  // recursive-like used to draw the segments: a new rectangle is
+  // generated whenever the length of a segment becomes lower than
+  // overlayRectSize(), or if the segment would be draw anyway
+  // to avoid strange things happening we impose that the distance
+  // in parameter space be less than a threshold before generating
+  // any overlay.
+  //
+  // The third parameter in workitem is a pointer into a stack of
+  // all generated rectangles (in real coordinate space); if 0
+  // there is no rectangles associated to that segment yet.
+  //
+  // Using the final mOverlay stack would be much more efficient, but
+  // 1. needs transformations into window space
+  // 2. would be more difficult to drop rectangles not intersecting
+  //    the window.
+  std::stack<Rect> overlaystack;
 
-  // first work on some 20 initial points and push them onto the
-  // stack.. :
-  coordparampair prev =
-    coordparampair( 0, conicGetCoord( 0, ecostheta0, esintheta0,
-                                      pdimen, focus1 ) );
-
-  for ( double i = M_PI/20; i <= 2*M_PI; i += M_PI/20 )
-  {
-    Coordinate c = conicGetCoord( i, ecostheta0, esintheta0,
-                                  pdimen, focus1 );
-    workstack.push( workitem( coordparampair( i, c ), prev ) );
-    prev = coordparampair( i, c );
-  };
+  // mp: the original version in which an initial set of 20 intervals
+  // were pushed onto the stack is replaced by a single interval and
+  // by forcing subdivision till h < hmax (with more or less the same
+  // final result).
+  // First push the [0,2*pi] interval into the stack:
+  workstack.push( workitem(
+                    coordparampair( 0, conicGetCoord( 0, ecostheta0, esintheta0,
+                                                      pdimen, focus1 ) ),
+                    coordparampair( 2*M_PI, conicGetCoord( 2*M_PI, ecostheta0, esintheta0,
+                                                           pdimen, focus1 ) ),
+                    0 ) );
 
   // maxlength is the square of the maximum size that we allow
   // between two points..
@@ -490,8 +544,11 @@ void KigPainter::drawConic( const ConicPolarEquationData& data )
   double sigma = maxlength/4;
   // distance between two parameter values cannot be too small
   double hmin = 1e-4;
+  // distance between two parameter values cannot be too large
+  double hmax = M_PI/20;
+  double hmaxoverlay = M_PI/4;
 
-  int count = 20;              // the number of points we've already
+  int count = 1;               // the number of segments we've already
                                // visited...
   static const int maxnumberofpoints = 1000;
 
@@ -511,19 +568,28 @@ void KigPainter::drawConic( const ConicPolarEquationData& data )
       Coordinate p1 = curitem.second.second;
       double t2 = ( curitem.first.first + curitem.second.first ) / 2;
       double h = fabs( curitem.second.first - curitem.first.first ) /2;
+      Rect *overlaypt = curitem.overlay;
       Coordinate p2 = conicGetCoord( t2, ecostheta0, esintheta0,
-                                    pdimen, focus1 );
+                                     pdimen, focus1 );
+      bool dooverlay = ! overlaypt && h < hmaxoverlay
+ && fabs( p0.x - p1.x ) <= overlayRectSize()
+ && fabs( p0.y - p1.y ) <= overlayRectSize();
       bool addn = sr.contains( p2 );
       // estimated error between the curve and the segments
       double errsq = (0.5*p0 + 0.5*p1 - p2).squareLength();
       errsq /= 4;
       curitemok = false;
-      if ( errsq < sigma || h < hmin )
+      bool dodraw = h < hmax && ( errsq < sigma || h < hmin );
+      if ( tNeedOverlay && ( dooverlay || dodraw ) )
+      {
+        Rect newoverlay( p0, p1 );
+        overlaystack.push( newoverlay );
+        overlaypt = &overlaystack.top();
+      }
+      if ( overlaypt ) overlaypt->setContains( p2 );
+      if ( dodraw )
       {
         // draw the two segments
-        overlay.setContains( p0 );
-        overlay.setContains( p1 );
-        overlay.setContains( p2 );
         QPoint tp0 = toScreen(p0);
         QPoint tp1 = toScreen(p1);
         QPoint tp2 = toScreen(p2);
@@ -533,19 +599,115 @@ void KigPainter::drawConic( const ConicPolarEquationData& data )
       else
       {
         // push into stack in order to process both subintervals
-        if (addn || sr.contains( p0 ) )
-          workstack.push( workitem( curitem.first, coordparampair( t2, p2 ) ) );
-        if (addn || sr.contains( p1 ) )
+        if (addn || sr.contains( p0 ) || h >= hmax)
+          workstack.push( workitem( curitem.first, coordparampair( t2, p2 ),
+                                    overlaypt ) );
+        if (addn || sr.contains( p1 ) || h >= hmax)
         {
-          curitem = workitem( coordparampair( t2, p2 ), curitem.second );
+          curitem = workitem( coordparampair( t2, p2 ), curitem.second ,
+                              overlaypt );
           curitemok = true;
         }
       }
     }
   }
 
-  mOverlay.push_back( toScreen( overlay ) );
+  assert ( tNeedOverlay || overlaystack.empty() );
+  if ( tNeedOverlay )
+  {
+    Rect border = window();
+    while ( ! overlaystack.empty() )
+    {
+      Rect overlay = overlaystack.top();
+      overlaystack.pop();
+      if (overlay.intersects( border ))
+        mOverlay.push_back( toScreenEnlarge( overlay ) );
+    }
+  }
   mNeedOverlay = tNeedOverlay;
+  pen.setCapStyle( Qt::FlatCap );
+  mP.setPen( pen );
+}
+
+void KigPainter::drawCubicRecurse (
+                   double& xleft, double& yleft, bool& validleft,
+                   int& numrootsleft,
+                   double &xright, double &yright, bool &validright,
+                   int &numrootsright,
+                   const CubicCartesianEquationData &data, int &root,
+                   double &ymin, double &ymax, double &tol,
+                   bool& tNeedOverlay, Rect& overlay)
+{
+  double ltol = tol;
+  if ( ! ( validleft && validright && numrootsleft == numrootsright ) )
+    // in questo caso la tolleranza e' molto minore
+    ltol /= 100;
+  if ( xright - xleft < ltol )
+  {
+    if ( validleft && validright && numrootsleft == numrootsright )
+    {
+      /* draw the segment */
+      Coordinate pleft = Coordinate( xleft, yleft );
+      Coordinate pright = Coordinate( xright, yright );
+      QPoint tpleft = toScreen(pleft);
+      QPoint tpright = toScreen(pright);
+      mP.drawLine( tpleft, tpright );
+    }
+  } else {
+    double xmiddle = (xright + xleft)/2;
+    bool validmiddle;
+    int numrootsmiddle;
+    double ymiddle = calcCubicYvalue ( xmiddle, ymin, ymax, root, data,
+                   validmiddle, numrootsmiddle );
+    Coordinate pmiddle = Coordinate( xmiddle, ymiddle );
+    if ( validmiddle && tNeedOverlay ) overlay.setContains( pmiddle );
+    drawCubicRecurse ( xleft, yleft, validleft, numrootsleft,
+                   xmiddle, ymiddle, validmiddle, numrootsmiddle,
+                   data, root, ymin, ymax, tol, tNeedOverlay, overlay );
+    drawCubicRecurse ( xmiddle, ymiddle, validmiddle, numrootsmiddle,
+                   xright, yright, validright, numrootsright,
+                   data, root, ymin, ymax, tol, tNeedOverlay, overlay );
+  }
+}
+
+void KigPainter::drawCubic( const CubicCartesianEquationData& data )
+{
+  // we manage our own overlay
+  bool tNeedOverlay = mNeedOverlay;
+  mNeedOverlay = false;
+  QPen pen = mP.pen();
+  pen.setCapStyle( Qt::RoundCap );
+  mP.setPen( pen );
+  Rect border = window();
+  Rect overlay;
+
+  double ymin = border.bottom();
+  double ymax = border.top();
+  bool validleft, validright;
+  int numrootsleft, numrootsright;
+
+  double xleft = border.left();
+  double xright = border.right();
+  double tol = (xright - xleft)/100;
+  for ( int root = 1; root <= 3; root++ )
+  {
+    double yleft = calcCubicYvalue ( xleft, ymin, ymax, root, data,
+                   validleft, numrootsleft );
+    double yright = calcCubicYvalue ( xright, ymin, ymax, root, data,
+                   validright, numrootsright );
+    Coordinate p = Coordinate( xleft, yleft );
+    if ( validleft && tNeedOverlay ) overlay.setContains( p );
+    p = Coordinate( xright, yright );
+    if ( validright && tNeedOverlay ) overlay.setContains( p );
+    drawCubicRecurse ( xleft, yleft, validleft, numrootsleft,
+                   xright, yright, validright, numrootsright,
+                   data, root, ymin, ymax, tol,
+                   tNeedOverlay, overlay);
+  }
+  if ( tNeedOverlay ) mOverlay.push_back( toScreen( overlay ) );
+  mNeedOverlay = tNeedOverlay;
+  pen.setCapStyle( Qt::FlatCap );
+  mP.setPen( pen );
 }
 
 void KigPainter::drawLine( const LineData& d )
