@@ -67,7 +67,7 @@ MType::MType( const QDomElement& e )
 {
   assert(e.tagName() == "MType");
   QString tmpAN = e.attribute("name");
-  assert(tmpAN);
+  assert(!tmpAN.isNull());
   mname = tmpAN;
   QDomNode n = e.firstChild();
   QDomElement el = n.toElement();
@@ -113,9 +113,9 @@ KAction* Type::constructAction( KigDocument* d )
   return a;
 }
 
-KigMode* MType::constructMode( NormalMode* mode, KigDocument* doc )
+KigMode* MType::constructMode( KigDocument& doc )
 {
-  return new StdConstructionMode( this, mode, doc );
+  return new StdConstructionMode( this, doc );
 }
 
 void deleteAction( KAction* a )
