@@ -31,7 +31,7 @@ class NormalMode
   : public KigMode
 {
 public:
-  NormalMode( KigDocument* );
+  NormalMode( KigDocument& );
   ~NormalMode();
   void leftClicked( QMouseEvent*, KigWidget* );
   void leftMouseMoved( QMouseEvent*, KigWidget* );
@@ -42,11 +42,6 @@ public:
   void rightClicked( QMouseEvent*, KigWidget* );
   void rightMouseMoved( QMouseEvent*, KigWidget* );
   void rightReleased( QMouseEvent*, KigWidget* );
-
-  /**
-   * The user is just moving around his cursor,
-   * without having clicked anywhere...
-   */
   void mouseMoved( QMouseEvent*, KigWidget* );
 
   /**
