@@ -54,7 +54,7 @@ bool Conic::contains (const Coordinate& o, const ScreenInfo& si ) const
   double esintheta0 = d.esintheta0;
   double pdimen = d.pdimen;
 
-  pos = Coordinate(o) - focus1;
+  pos = o - focus1;
   len = pos.length();
   costheta = pos.x / len;
   sintheta = pos.y / len;
@@ -70,6 +70,7 @@ bool Conic::contains (const Coordinate& o, const ScreenInfo& si ) const
 void Conic::draw (KigPainter& p, bool ss) const
 {
   p.setPen(QPen ( selected && ss ? Qt::red : mColor, 1));
+  p.setWidth ( mWidth );
   p.setBrush( Qt::NoBrush );
 
   p.drawConic( polarEquationData() );
@@ -137,12 +138,12 @@ EllipseBFFP::EllipseBFFP(const EllipseBFFP& c)
 
 const QCString Conic::sBaseTypeName()
 {
-  return I18N_NOOP("conic");
+  return I18N_NOOP("Conic");
 }
 
 const QString EllipseBFFP::sDescriptiveName()
 {
-  return i18n("Ellipse by focuses and point");
+  return i18n("Ellipse by Focuses && Point");
 }
 
 const QString EllipseBFFP::sDescription()
@@ -208,16 +209,6 @@ void ConicBFFP::sDrawPrelimCommon( KigPainter& p, const Objects& args, int type)
   p.drawConic( data );
 }
 
-Conic* Conic::toConic()
-{
-  return this;
-}
-
-const Conic* Conic::toConic() const
-{
-  return this;
-}
-
 void ConicB5P::calc()
 {
   std::vector<Coordinate> points;
@@ -257,7 +248,7 @@ ConicB5P::ConicB5P(const ConicB5P& c)
 
 const QString ConicB5P::sDescriptiveName()
 {
-  return i18n("Conic by five points");
+  return i18n("Conic by Five Points");
 }
 
 const QString ConicB5P::sDescription()
@@ -352,7 +343,7 @@ ParabolaBTP::ParabolaBTP(const ParabolaBTP& c)
 
 const QString ParabolaBTP::sDescriptiveName()
 {
-  return i18n( "Vertical parabola by three points" );
+  return i18n( "Vertical Parabola by Three Points" );
 }
 
 const QString ParabolaBTP::sDescription()
@@ -414,11 +405,6 @@ EllipseBFFP::~EllipseBFFP()
 {
 }
 
-EllipseBFFP* EllipseBFFP::copy()
-{
-  return new EllipseBFFP(*this);
-}
-
 const QCString EllipseBFFP::sFullTypeName()
 {
   return "EllipseBFFP";
@@ -428,11 +414,6 @@ HyperbolaBFFP::~HyperbolaBFFP()
 {
 }
 
-HyperbolaBFFP* HyperbolaBFFP::copy()
-{
-  return new HyperbolaBFFP(*this);
-}
-
 ConicBFFP::~ConicBFFP()
 {
 }
@@ -500,7 +481,7 @@ HyperbolaBFFP::HyperbolaBFFP( const HyperbolaBFFP& c )
 
 const QString HyperbolaBFFP::sDescriptiveName()
 {
-  return i18n("Hyperbola by focuses and point");
+  return i18n("Hyperbola by Focuses && Point");
 }
 
 const QString HyperbolaBFFP::sDescription()
@@ -554,16 +535,16 @@ int Conic::conicType() const
   return 0;
 }
 
-QString Conic::type() const
+QString Conic::conicTypeString() const
 {
   switch (conicType())
   {
   case 1:
-    return I18N_NOOP("ellipse");
+    return i18n("Ellipse");
   case -1:
-    return I18N_NOOP("hyperbola");
+    return i18n("Hyperbola");
   case 0:
-    return I18N_NOOP("parabola");
+    return i18n("Parabola");
   default:
     assert( false );
   }
@@ -599,9 +580,10 @@ const Property Conic::property( uint which, const KigWidget& w ) const
 {
   int pnum = 0;
 
-  if ( which < Curve::numberOfProperties() ) return Curve::property( which, w );
+  if ( which < Curve::numberOfProperties() )
+    return Curve::property( which, w );
   if ( which == Curve::numberOfProperties() + pnum++ )
-    return Property( type() );
+    return Property( conicTypeString() );
   else if ( which == Curve::numberOfProperties() + pnum++ )
     return Property( focus1() );
   else if ( which == Curve::numberOfProperties() + pnum++ )
@@ -617,10 +599,10 @@ const QCStringList Conic::properties() const
 {
   QCStringList l = Curve::properties();
   l << I18N_NOOP( "Type" );
-  l << I18N_NOOP( "First focus" );
-  l << I18N_NOOP( "Second focus" );
-  l << I18N_NOOP( "Cartesian equation" );
-  l << I18N_NOOP( "Polar equation" );
+  l << I18N_NOOP( "First Focus" );
+  l << I18N_NOOP( "Second Focus" );
+  l << I18N_NOOP( "Cartesian Equation" );
+  l << I18N_NOOP( "Polar Equation" );
   assert( l.size() == Conic::numberOfProperties() );
   return l;
 }
@@ -725,7 +707,7 @@ EquilateralHyperbolaB4P::EquilateralHyperbolaB4P(const EquilateralHyperbolaB4P&
 
 const QString EquilateralHyperbolaB4P::sDescriptiveName()
 {
-  return i18n( "Equilateral hyperbola by four points" );
+  return i18n( "Equilateral Hyperbola by Four Points" );
 }
 
 const QString EquilateralHyperbolaB4P::sDescription()
@@ -828,7 +810,7 @@ ConicBAAP::ConicBAAP(const ConicBAAP& c)
 
 const QString ConicBAAP::sDescriptiveName()
 {
-  return i18n( "Hyperbola by asymptotes and point" );
+  return i18n( "Hyperbola by Asymptotes && Point" );
 }
 
 const QString ConicBAAP::sDescription()
@@ -959,7 +941,7 @@ ConicBDFP::ConicBDFP(const ConicBDFP& c)
 
 const QString ConicBDFP::sDescriptiveName()
 {
-  return i18n("Conic by directrix, focus and point");
+  return i18n("Conic by Directrix, Focus && Point");
 }
 
 const QString ConicBDFP::sDescription()
@@ -1076,3 +1058,8 @@ const ConicCartesianEquationData ConicBDFP::cartesianEquationData() const
 {
   return ConicCartesianEquationData( pequation );
 }
+
+bool Conic::isa( int type ) const
+{
+  return type == ConicT ? true : Parent::isa( type );
+}
