@@ -120,10 +120,9 @@ bool TextLabel::contains( const Coordinate& o, const ScreenInfo& ) const
   return mrwdoi.contains( o );
 }
 
-KigMode* TextLabel::sConstructMode( Type*, KigDocument* d,
-                                    NormalMode* p )
+KigMode* TextLabel::sConstructMode( Type*, KigDocument& d )
 {
-  return new TextLabelConstructionMode( p, d );
+  return new TextLabelConstructionMode( d );
 }
 
 void TextLabel::startMove( const Coordinate& c, const ScreenInfo& )
@@ -137,21 +136,6 @@ void TextLabel::moveTo( const Coordinate& c )
   mpwwmt = c;
 }
 
-TextLabel* TextLabel::copy()
-{
-  return new TextLabel( *this );
-}
-
-const TextLabel* TextLabel::toTextLabel() const
-{
-  return this;
-}
-
-TextLabel* TextLabel::toTextLabel()
-{
-  return this;
-}
-
 const QCString TextLabel::vBaseTypeName() const
 {
   return sBaseTypeName();
@@ -159,7 +143,7 @@ const QCString TextLabel::vBaseTypeName() const
 
 QCString TextLabel::sBaseTypeName()
 {
-  return I18N_NOOP("label");
+  return I18N_NOOP("Label");
 }
 
 const QCString TextLabel::vFullTypeName() const
@@ -179,7 +163,7 @@ const QString TextLabel::vDescriptiveName() const
 
 const QString TextLabel::sDescriptiveName()
 {
-  return i18n( "A text label" );
+  return i18n( "Text Label" );
 }
 
 const QCString TextLabel::vIconFileName() const
@@ -243,7 +227,7 @@ const QString TextLabel::vDescription() const
 }
 const QString TextLabel::sDescription()
 {
-  return i18n("A piece of text you can add to the document..");
+  return i18n("A piece of text you can add to the document.");
 }
 
 TextLabel::TextLabel( const QString text, const Coordinate c, const propvect& props )
@@ -278,3 +262,8 @@ const QCStringList TextLabel::properties() const
   assert( l.size() == TextLabel::numberOfProperties() );
   return l;
 }
+
+bool TextLabel::isa( int type ) const
+{
+  return type == TextLabelT ? true : Parent::isa( type );
+}
