@@ -16,8 +16,8 @@
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 // 02111-1307, USA.
 
-#ifndef LABEL_H
-#define LABEL_H
+#ifndef KIG_OBJECTS_LABEL_H
+#define KIG_OBJECTS_LABEL_H
 
 #include "object.h"
 
@@ -29,6 +29,7 @@
 class TextLabel
   : public Object
 {
+  typedef Object Parent;
 public:
   typedef std::vector<TextLabelProperty> propvect;
 
@@ -37,14 +38,11 @@ public:
   TextLabel( const QString text, const Coordinate c, const propvect& props );
   ~TextLabel();
 
-  TextLabel* copy();
+  virtual bool isa( int type ) const;
 
   prop_map getParams ();
   void setParams ( const prop_map& );
 
-  const TextLabel* toTextLabel() const;
-  TextLabel* toTextLabel();
-
   const QCString vBaseTypeName() const;
   static QCString sBaseTypeName();
 
@@ -72,8 +70,7 @@ public:
   void calc();
   void calcForWidget( const KigWidget& w );
 
-  static KigMode* sConstructMode( Type* t, KigDocument* d,
-                                  NormalMode* p );
+  static KigMode* sConstructMode( Type* t, KigDocument& d );
 
   Objects getParents() const;
 
