@@ -20,8 +20,6 @@
 
 #include "segment.h"
 
-#include "../misc/kigpainter.h"
-
 #include <klocale.h>
 
 MidPoint::MidPoint(const MidPoint& m)
@@ -36,13 +34,13 @@ MidPoint::~MidPoint()
 {
 }
 
-void MidPoint::startMove(const Coordinate& p)
+void MidPoint::startMove(const Coordinate& p, const ScreenInfo& si)
 {
-  if (contains(p,false) && p1 )
+  if (contains(p, si) && p1 )
   {
     assert( p1 && p2 );
     howm = howmMoving;
-    p2->startMove(p2->getCoord());
+    p2->startMove(p2->getCoord(), si);
   }
   else howm = howmFollowing;
 }
@@ -59,7 +57,7 @@ void MidPoint::stopMove()
 {
 };
 
-void MidPoint::calc( const ScreenInfo& )
+void MidPoint::calc()
 {
   if ( p1 || p2 )
   {
@@ -169,5 +167,5 @@ void MidPoint::sDrawPrelim( KigPainter& p, const Objects& args )
     Coordinate a = p->getCoord(), b = q->getCoord();
     m = ( a + b ) / 2;
   };
-  p.drawPoint( m, false );
+  sDrawPrelimPoint( p, m );
 }
