@@ -23,21 +23,17 @@
 
 #include "point.h"
 
-class Arc;
-
 class RotatedPoint
   : public Point
 {
   Point* mp;
   Point* mc;
-  Arc* ma;
+  Angle* ma;
 public:
   RotatedPoint( const Objects& os );
   RotatedPoint( const RotatedPoint& p );
   ~RotatedPoint();
 
-  virtual RotatedPoint* copy();
-
   const QCString vFullTypeName() const;
   static const QCString sFullTypeName();
   const QString vDescriptiveName() const;
