@@ -45,25 +45,10 @@ Vector::Vector( const Vector& s )
   p2->addChild(this);
 }
 
-Vector* Vector::copy()
-{
-  return new Vector( * this );
-}
-
 Vector::~Vector()
 {
 };
 
-Vector* Vector::toVector()
-{
-  return this;
-}
-
-const Vector* Vector::toVector() const
-{
-  return this;
-}
-
 const QCString Vector::vBaseTypeName() const
 {
   return sBaseTypeName();
@@ -71,7 +56,7 @@ const QCString Vector::vBaseTypeName() const
 
 QCString Vector::sBaseTypeName()
 {
-  return I18N_NOOP( "vector" );
+  return I18N_NOOP( "Vector" );
 }
 
 const QCString Vector::vFullTypeName() const
@@ -101,7 +86,7 @@ const QString Vector::vDescription() const
 
 const QString Vector::sDescription()
 {
-  return i18n( "Construct a vector from two given points.." );
+  return i18n( "Construct a vector from two given points." );
 }
 
 const QCString Vector::vIconFileName() const
@@ -139,6 +124,7 @@ void Vector::draw( KigPainter& p, bool ss ) const
   Coordinate a = p1->getCoord();
   Coordinate b = p2->getCoord();
   p.setPen( QPen( selected && ss ? Qt::red : mColor, 1 ));
+  p.setWidth ( mWidth );
   drawVector( p, a, b );
 };
 
@@ -223,3 +209,8 @@ const Coordinate Vector::getDir() const
 {
   return getP2() - getP1();
 }
+
+bool Vector::isa( int type ) const
+{
+  return type == VectorT ? true : Parent::isa( type );
+}
